import _defineProperty from '@babel/runtime/helpers/defineProperty';
import { parse } from 'query-string';

class LocaleResolver {
  constructor(detectors, transformers = []) {
    _defineProperty(this, "locales", void 0);

    _defineProperty(this, "transformers", void 0);

    this.locales = detectors.map(detector => {
      return detector.getLocales();
    }).reduce((a, b) => a.concat(b), []);
    this.transformers = transformers;
  }

  getLocales(transformers = []) {
    let {
      locales
    } = this;
    [...this.transformers, ...transformers].forEach(transformer => {
      locales = transformer.transform(locales);
    });
    return locales.filter((value, index, array) => {
      return array.indexOf(value) === index;
    });
  }

}

/**
 * Detector classes obtain list of user's locales. It can be from browser, url, cookies, storage, ... whatever you want
 * Get locales method always return array of locales.
 */
class Detector {}

/**
 * Navigator detector try load locales from browser navigator variable.
 */

class NavigatorDetector extends Detector {
  /**
   * On default languages are loaded from window.navigator
   *
   * @param {Navigator} navigator - browser navigator variable
   */
  constructor(navigator = window.navigator) {
    super();

    _defineProperty(this, "navigator", void 0);

    this.navigator = navigator;
  }
  /**
   * Get list of locales.
   *
   * @returns {Array<string>} list of locales from browser
   */


  getLocales() {
    // @ts-ignore
    const {
      languages,
      language,
      userLanguage
    } = this.navigator;

    if (Array.isArray(languages)) {
      return languages;
    }

    if (language) {
      return [language];
    }

    if (userLanguage) {
      return [userLanguage];
    }

    return [];
  }

}

/**
 * This detector load locales from GET parameters. The value should be locale separated list.
 */

class UrlDetector extends Detector {
  /**
   * Constructor
   *
   * @param {string} parameter - Which parameter to look for.
   * @param {Location} location - window.location is used on default
   */
  constructor(parameter, location = window.location) {
    super();

    _defineProperty(this, "location", void 0);

    _defineProperty(this, "parameter", void 0);

    this.location = location;
    this.parameter = parameter;
  }
  /**
   * Get list of locales from browser url.
   *
   * @returns {Array<string>} list of locales
   */


  getLocales() {
    const locales = parse(this.location.search)[this.parameter] || null;

    if (typeof locales === 'string') {
      return locales.split(',');
    }

    return [];
  }

}

/**
 * With FromListDetector you can set locales by your own.
 */

class FromListDetector extends Detector {
  /**
   * @param {Array<string>} locales - list of locales
   */
  constructor(locales) {
    super();

    _defineProperty(this, "locales", void 0);

    this.locales = locales;
  }
  /**
   * Get list your of locales.
   *
   * @returns {Array<string>} list of your locales
   */


  getLocales() {
    return this.locales;
  }

}



var index = {
    __proto__: null,
    Detector: Detector,
    NavigatorDetector: NavigatorDetector,
    UrlDetector: UrlDetector,
    FromListDetector: FromListDetector
};

/**
 * Transformer are used to change format of locale (eg. convert it to uppercase), create fallbacks or delete invalid items.
 */
class Transformer {}

/**
 * This transformers create fallbacks from complex locales to simpliest.
 * 'en-US-east' => ['en-US-east', 'en-US', 'en']
 * 'es-ES' => ['es-ES', 'es']
 */

class FallbacksTransformer extends Transformer {
  /**
   * Add fallbacks to locales. Locales should be in IETF language tag format.
   *
   * @param {Array<string>} locales - list of locales
   * @returns {Array<string>} locales with fallbacks
   */
  transform(locales) {
    return locales.map(locale => {
      const splitedLocale = locale.split('-');
      return splitedLocale.map((value, index) => {
        const localeGenerator = [];

        for (let i = 0; i <= index; i++) {
          localeGenerator.push(splitedLocale[i]);
        }

        return localeGenerator.join('-');
      }).reverse();
    }).reduce((a, b) => a.concat(b), []);
  }

}

/**
 * This transformer convert locales to standard IETF language tag.
 */

class IETFTransformer extends Transformer {
  /**
   * Constructor.
   *
   * @param {string} localeSeparator - how is locales parts separated
   */
  constructor(localeSeparator = '-') {
    super();

    _defineProperty(this, "localeSeparator", void 0);

    this.localeSeparator = localeSeparator;
  }
  /**
   * Convert locales to standard IETF language tag.
   *
   * @param {Array<string>} locales - list of locales
   * @returns {Array<string>} list of transformed locales
   */


  transform(locales) {
    return locales.map(locale => {
      return locale.split(this.localeSeparator).map((value, index) => {
        return index === 1 ? value.toUpperCase() : value.toLowerCase();
      }).join('-');
    });
  }

}

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }
const defaultConvertTable = {
  'es-XL': 'es-419',
  hindi: 'hi',
  indonesian: 'id',
  english: 'en'
};
/**
 * It convert invalid locales.
 */

class InvalidLocalesTransformer extends Transformer {
  /**
   * Constructor.
   *
   * @param {SimpleObject} convertTable - simple convert table
   */
  constructor(convertTable = {}) {
    super();

    _defineProperty(this, "convertTable", void 0);

    this.convertTable = _objectSpread(_objectSpread({}, defaultConvertTable), convertTable);
  }
  /**
   * Transform invalid languages.
   *
   * @param {Array<string>} locales - list of locales
   * @returns {Array<string>} cleaned list of locales
   */


  transform(locales) {
    return locales.map(locale => {
      return this.convertTable[locale] || locale;
    });
  }

}

/**
 * Transform all locales to languages only with region variant.
 */

class LanguageOnlyTransformer extends Transformer {
  /**
   * Transform locales to languages only with region variant.
   *
   * @param {Array<string>} locales - list of locales
   * @returns {Array<string>} - list of languages
   */
  transform(locales) {
    return locales.map(locale => {
      return locale.split('-')[0];
    });
  }

}

/**
 * This transformer allows you to append default locale.
 */

class DefaultLocaleTransformer extends Transformer {
  /**
   * Constructor.
   *
   * @param {string} defaultLocale - set default locale
   */
  constructor(defaultLocale) {
    super();

    _defineProperty(this, "defaultLocale", void 0);

    this.defaultLocale = defaultLocale;
  }
  /**
   * Add default locale to end of array
   *
   * @param {Array<string>} locales - list of locales
   * @returns {Array<string>} locales with default
   */


  transform(locales) {
    const transformedLocales = [...locales.map(locale => {
      const splitedLocale = locale.split('-');
      return splitedLocale.map((value, index) => {
        const localeGenerator = [];

        for (let i = 0; i <= index; i++) {
          localeGenerator.push(splitedLocale[i]);
        }

        return localeGenerator.join('-');
      }).reverse();
    }).reduce((a, b) => a.concat(b), []), ...[this.defaultLocale]]; // slice locales that comes after default locale

    return transformedLocales.slice(0, transformedLocales.indexOf(this.defaultLocale) + 1);
  }

}

/**
 * This transformer allows you to filter locales
 */

class AllowOnlyTransformer extends Transformer {
  /**
   * Constructor.
   *
   * @param {Array<string>} allowedLocales - list of allowed locales
   */
  constructor(allowedLocales) {
    super();

    _defineProperty(this, "allowedLocales", void 0);

    this.allowedLocales = allowedLocales;
  }
  /**
   * Return only allowed locales.
   *
   * @param {Array<string>} locales - list of locales
   * @returns {Array<string>} allowed locales
   */


  transform(locales) {
    return locales.filter(locale => {
      return this.allowedLocales.includes(locale);
    });
  }

}



var index$1 = {
    __proto__: null,
    Transformer: Transformer,
    FallbacksTransformer: FallbacksTransformer,
    IETFTransformer: IETFTransformer,
    InvalidLocalesTransformer: InvalidLocalesTransformer,
    LanguageOnlyTransformer: LanguageOnlyTransformer,
    DefaultLocaleTransformer: DefaultLocaleTransformer,
    AllowOnlyTransformer: AllowOnlyTransformer
};

export { index as DETECTORS, LocaleResolver, index$1 as TRANSFORMERS };
